#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later

"""
Check out mkosi into specified location.
With -u, if changed, commit the latest hash.
"""

import argparse
import re
import shlex
import subprocess
from pathlib import Path

URL = 'https://github.com/systemd/mkosi'
BRANCH = 'main'  # We only want to ever use commits on upstream 'main' branch
CONFIG = Path('mkosi/mkosi.conf')
WORKFLOWS = [
    Path('.github/workflows') / f for f in ['mkosi.yml', 'coverage.yml', 'linter.yml', 'unit-tests.yml']
]


def parse_args():
    p = argparse.ArgumentParser(
        description=__doc__,
    )
    p.add_argument(
        'dir',
        type=Path,
    )
    p.add_argument(
        '--update',
        '-u',
        action='store_true',
        default=False,
    )
    return p.parse_args()


def read_config():
    print(f'Reading {CONFIG}…')
    c = CONFIG.read_text()
    matches = [
        m.group(1) for m in re.finditer('^\s*MinimumVersion=commit:([a-z0-9]{40})\s*$', c, re.MULTILINE)
    ]
    assert len(matches) == 1
    return matches[0]


def commit_file(files: list[Path], commit: str, changes: str):
    message = '\n'.join((f'mkosi: update mkosi ref to {commit}', '', changes))

    cmd = ['git', 'commit', '-m', message, *(str(file) for file in files)]
    print(f'+ {shlex.join(cmd)}')
    subprocess.check_call(cmd)


def checkout_mkosi(args):
    if args.dir.exists():
        print(f'{args.dir} already exists.')
        return

    cmd = [
        'git',
        'clone',
        URL,
        f'--branch={BRANCH}',
        args.dir.as_posix(),
    ]
    print(f'+ {shlex.join(cmd)}')
    subprocess.check_call(cmd)


def update_mkosi(args):
    old_commit = read_config()

    cmd = ['git', '-C', args.dir.as_posix(), 'rev-parse', f'refs/remotes/origin/{BRANCH}']
    print(f'+ {shlex.join(cmd)}')
    new_commit = subprocess.check_output(cmd, text=True).strip()

    if old_commit == new_commit:
        print(f'mkosi: commit {new_commit!s} is still fresh')
        return

    cmd = [
        'git',
        '-C', args.dir.as_posix(),
        'log',
        '--graph',
        '--no-merges',
        '--pretty=oneline',
        '--no-decorate',
        '--abbrev-commit',
        '--abbrev=10',
        f'{old_commit}..{new_commit}',
    ]  # fmt: skip
    print(f'+ {shlex.join(cmd)}')
    changes = subprocess.check_output(cmd, text=True).strip()

    for f in [CONFIG, *WORKFLOWS]:
        s = f.read_text()
        assert old_commit in s
        print(f'mkosi: {f}: found old hash, updating…')
        new = s.replace(old_commit, new_commit)
        assert new != s
        f.write_text(new)

    commit_file([CONFIG, *WORKFLOWS], new_commit, changes)


if __name__ == '__main__':
    args = parse_args()
    checkout_mkosi(args)
    if args.update:
        update_mkosi(args)



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later

import filecmp
import os
import stat
import subprocess
import sys
import tempfile

OUTFILE_HEADER = '''#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# create-sys-script.py
#
# © 2017 Canonical Ltd.
# Author: Dan Streetman <dan.streetman@canonical.com>
'''

# Use this only to (re-)create the test/sys-script.py script,
# after adding or modifying anything in the test/sys/ directory


OUTFILE_MODE = 0o775

OUTFILE_FUNCS = r'''
import os, sys
import shutil

def d(path, mode):
    os.mkdir(path, mode)

def l(path, src):
    os.symlink(src, path)

def f(path, mode, contents):
    with open(path, "wb") as f:
        f.write(contents)
    os.chmod(path, mode)
'''

OUTFILE_MAIN = '''
if len(sys.argv) < 2:
    exit("Usage: {} <target dir>".format(sys.argv[0]))

if not os.path.isdir(sys.argv[1]):
    exit("Target dir {} not found".format(sys.argv[1]))

os.chdir(sys.argv[1])

if os.path.exists('sys'):
    shutil.rmtree('sys')
'''


def handle_dir(outfile, path):
    m = os.lstat(path).st_mode & 0o777
    outfile.write(f"d('{path}', {m:#o})\n")


def handle_link(outfile, path):
    src = os.readlink(path)
    outfile.write(f"l('{path}', '{src}')\n")


def escape_single_quotes(b):
    # remove the b'' wrapping each line repr
    r = repr(b)[2:-1]
    # python escapes all ' only if there are ' and " in the string
    if '"' not in r:
        r = r.replace("'", r'\'')
    # return line with all ' escaped
    return r


def handle_file(outfile, path):
    m = os.lstat(path).st_mode & 0o777
    with open(path, 'rb') as f:
        b = f.read()
    if b.count(b'\n') > 1:
        r = '\n'.join(escape_single_quotes(line) for line in b.split(b'\n'))
        r = f"b'''{r}'''"
    else:
        r = repr(b)
    outfile.write(f"f('{path}', {m:#o}, {r})\n")


def process_sysdir(outfile):
    for dirpath, dirnames, filenames in os.walk('sys'):
        handle_dir(outfile, dirpath)
        for d in dirnames:
            path = os.path.join(dirpath, d)
            if stat.S_ISLNK(os.lstat(path).st_mode):
                handle_link(outfile, path)
        for f in filenames:
            path = os.path.join(dirpath, f)
            mode = os.lstat(path).st_mode
            if stat.S_ISLNK(mode):
                handle_link(outfile, path)
            elif stat.S_ISREG(mode):
                handle_file(outfile, path)


def verify_dir(tmpd, path_a):
    path_b = os.path.join(tmpd, path_a)
    mode_a = os.lstat(path_a).st_mode
    mode_b = os.lstat(path_b).st_mode
    if not stat.S_ISDIR(mode_b):
        raise Exception('Not directory')
    if (mode_a & 0o777) != (mode_b & 0o777):
        raise Exception('Permissions mismatch')


def verify_link(tmpd, path_a):
    path_b = os.path.join(tmpd, path_a)
    if not stat.S_ISLNK(os.lstat(path_b).st_mode):
        raise Exception('Not symlink')
    if os.readlink(path_a) != os.readlink(path_b):
        raise Exception('Symlink dest mismatch')


def verify_file(tmpd, path_a):
    path_b = os.path.join(tmpd, path_a)
    mode_a = os.lstat(path_a).st_mode
    mode_b = os.lstat(path_b).st_mode
    if not stat.S_ISREG(mode_b):
        raise Exception('Not file')
    if (mode_a & 0o777) != (mode_b & 0o777):
        raise Exception('Permissions mismatch')
    if not filecmp.cmp(path_a, path_b, shallow=False):
        raise Exception('File contents mismatch')


def verify_script(tmpd):
    any = False
    for dirpath, dirnames, filenames in os.walk('sys'):
        any = True
        try:
            path = dirpath
            verify_dir(tmpd, path)
            for d in dirnames:
                path = os.path.join(dirpath, d)
                if stat.S_ISLNK(os.lstat(path).st_mode):
                    verify_link(tmpd, path)
                for f in filenames:
                    path = os.path.join(dirpath, f)
                    mode = os.lstat(path).st_mode
                    if stat.S_ISLNK(mode):
                        verify_link(tmpd, path)
                    elif stat.S_ISREG(mode):
                        verify_file(tmpd, path)
        except Exception:
            print(f'FAIL on "{path}"', file=sys.stderr)
            raise
    if not any:
        exit('Nothing found!')


if __name__ == '__main__':
    if len(sys.argv) < 2:
        exit('Usage: create-sys-script.py /path/to/test/')

    outfile = os.path.abspath(os.path.dirname(sys.argv[0]) + '/sys-script.py')
    print(f'Creating {outfile} using contents of {sys.argv[1]}/sys')

    os.chdir(sys.argv[1])

    with open(outfile, 'w') as f:
        os.chmod(outfile, OUTFILE_MODE)
        f.write(OUTFILE_HEADER.replace(os.path.basename(sys.argv[0]), os.path.basename(outfile)))
        f.write(OUTFILE_FUNCS)
        f.write(OUTFILE_MAIN)
        process_sysdir(f)

    with tempfile.TemporaryDirectory() as tmpd:
        print(f'Recreating sys/ using {outfile} at {tmpd}')
        subprocess.check_call([outfile, tmpd])
        verify_script(tmpd)

    print(f'Verification successful, {outfile} is correct')



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later
# pylint: disable=broad-except

import logging
import sys

import pexpect


def main():
    # TODO: drop once https://bugs.debian.org/1075733 is fixed
    with open('/usr/lib/os-release') as f:
        for line in f:
            if line.startswith('ID='):
                if 'debian' in line or 'ubuntu' in line:
                    sys.exit(77)

    logger = logging.getLogger('test-shutdown')

    consoles = []
    for _ in range(2):
        # Use script to allocate a separate pseudo tty to run the login shell in.
        console = pexpect.spawn(
            'script',
            ['--quiet', '--return', '--flush', '--command', 'login -f root', '/dev/null'],
            logfile=sys.stdout,
            env={'TERM': 'dumb'},
            encoding='utf-8',
            timeout=60,
        )

        logger.info('waiting for login prompt')
        console.expect('.*# ', 10)

        consoles += [console]

    consoles[1].sendline('tty')
    consoles[1].expect(r'/dev/(pts/\d+)')
    pty = console.match.group(1)
    logger.info('window 1 at tty %s', pty)

    logger.info('schedule reboot')
    consoles[1].sendline('shutdown -r')
    consoles[1].expect("Reboot scheduled for (?P<date>.*), use 'shutdown -c' to cancel", 2)
    date = consoles[1].match.group('date')
    logger.info('reboot scheduled for %s', date)

    logger.info('verify broadcast message')
    consoles[0].expect(f'Broadcast message from root@H on {pty}', 2)
    consoles[0].expect(f'The system will reboot at {date}!', 2)

    logger.info('check show output')
    consoles[1].sendline('shutdown --show')
    consoles[1].expect(f"Reboot scheduled for {date}, use 'shutdown -c' to cancel", 2)

    logger.info('cancel shutdown')
    consoles[1].sendline('shutdown -c')
    consoles[0].expect('System shutdown has been cancelled', 2)

    consoles[0].sendline('> /testok')


if __name__ == '__main__':
    main()

# vim: sw=4 et



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later

from csv import reader

# pylint: disable=consider-using-with


def read_table(filename):
    table = list(reader(open(filename, newline='')))
    table = table[1:]  # Skip header
    table.sort(key=lambda x: x[1])

    for row in table:
        # Some IDs end with whitespace, while they didn't in the old HTML table, so it's probably
        # a mistake, strip it.
        print(f'\nacpi:{row[1].strip()}*:\n ID_VENDOR_FROM_DATABASE={row[0].strip()}')


print('''\
# This file is part of systemd.
#
# Data imported from:
#     https://uefi.org/uefi-pnp-export
#     https://uefi.org/uefi-acpi-export''')

read_table('acpi_id_registry.csv')
read_table('pnp_id_registry.csv')



#!/usr/bin/python3
# SPDX-License-Identifier: LGPL-2.1-or-later

import socket
import sys

data = sys.stdin.buffer.read()
s = socket.create_connection(('localhost', 12345), timeout=15)
s.settimeout(15)
s.sendall(data)
received = b''
while len(received) < len(data):
    chunk = s.recv(65536)
    if not chunk:
        break
    received += chunk
sys.stdout.buffer.write(received)
s.close()
